/**
 * user.js
 * GraphQL Type definitions using schema language
 */
'use strict';

const userTypeDefs = `

# A user.
type User implements Node {

  # User's identifier.
  id: ID!

  # User's login name.
  userName: String!

  # User's last name.
  lastName: String

  # User's first name.
  firstName: String

  # User's email.
  email: String

  # User's phone number.
  phone: String

  # User's mobile phone number.
  mobile: String

  # User's pager number.
  pager: String

  # User's fax number.
  fax: String

  # User's company.
  company: String

  # User's department.
  department: String

  # User's office.
  office: String

  # User's job title.
  jobTitle: String

  # User's employee number.
  employeeNumber: String

  # User's employee type.
  employeeType: String

  # User's manager.
  manager: User

  # User's personal title.
  personalTitle: String

  # User's initials.
  initials: String

  # User's postal address.
  postalAddress: String

  # User's postal code.
  postalCode: String

  # User's city.
  city: String

  # User's state.
  state: String

  # User's country.
  country: String

  # Is the user a process supervisor?
  isSupervisor: Boolean!

  # Is the user a process folder manager?
  isProcessFolderManager: Boolean!

  # Is the user an administrator?
  isAdministrator: Boolean!

  # Is the user's account active?
  isActive: Boolean!

  # Is the user's account archived?
  isArchived: Boolean!

  # Is the user the viewing user?
  isViewer: Boolean!
  
  # List of processes available to the user.
  processes (page: Page,  orderBy: ProcessListOrder, filter: ProcessFilter, onBehalfOf: ID): ProcessListPage!

  # List of processes that the user can launch. 
  launchableProcesses (page: Page, orderBy: ProcessListOrder, filter: LaunchableProcessFilter, onBehalfOf: ID): ProcessListPage!

  # List of requests.
  requests (page: Page, orderBy: RequestListOrder, filter: RequestFilter, onBehalfOf: ID): RequestListPage!

  # List of actions.
  actions (page: Page, orderBy: ActionListOrder, filter: ActionRequestFilter, onBehalfOf: ID): ActionListPage!

  # List of views available to the user.
  views (page: Page, orderBy: ViewListOrder, filter: ViewFilter): ViewListPage!

  # List of delegations managed by the user.
  delegations (page: Page, orderBy: DelegationListOrder, filter: DelegationFilter): DelegationListPage!

  # List of the user's delegators.
  delegators (page: Page): UserListPage!

  # List of the user's delegable process participants.
  delegableProcessParticipants(page: Page, orderBy: DelegableProcessParticipantOrder,filter: DelegableProcessParticipantFilter): DelegableProcessParticipantListPage!

  # List of categories available to the user.
  categories (page: Page,  orderBy: CategoryListOrder, filter: CategoryFilter, onBehalfOf: ID): CategoryListPage!
}

# A paginated list of users.
type UserListPage {

  # Identifies the total count of items in the page.
  totalCount: Int!

  # Indicates if there are more pages to fetch.
  hasNextPage: Boolean!

  # Indicates if there are any pages prior to the current page.
  hasPreviousPage: Boolean!

  # A list of users.
  items: [User]
}

# A user list order.
input UserListOrder {

  # The field to order users by.
  field: UserListOrderField

  # Ordering direction.
  direction: ListOrderDirection
}

# A list of fields to order a user list.
enum UserListOrderField {

  # Order users by username.
  USERNAME

  # Order users by last name.
  LASTNAME
}

# A filter on users.
input UserFilter {

  # Filter users by name.
  name: String

  # Filter users by department.
  department: String

  # Filter users by directory name.
  directoryName: String
}

`;

module.exports = userTypeDefs;
