/**
 * server.js
 * Server initialization
 */
'use strict';
const express = require('express');
const cors = require('cors');
const graphqlExpress = require('express-graphql');
const schemaPrinter = require('graphql/utilities/schemaPrinter');
const { createSchema } = require('./models/schema');
const uuidV1 = require('uuid/v1');
const WorkflowService = require('./services/workflow/workflowservice');
const { logger } = require('./utils/logger.js');
const { getAuthUserName, getWorkflowContext } = require('./models/helpers/context');
const { createLoaders } = require('./models/loaders/');
const config = require('./config');

const rootUrl = '/*/graphql';
const app = express();

logger('INFO', 'Application NODE_ENV: ' + process.env.NODE_ENV);
logger('INFO', 'GraphQL debug enabled: ' + config.graphql.debugEnabled);
logger('INFO', 'Workflow service debug enabled: ' + config.workflowService.debugEnabled);

// Schema creation
const schema = createSchema(config.graphql);

// Workflow service creation
const workflowService = new WorkflowService(config.workflowService);
workflowService.start().then(
  result => logger('INFO', 'Workflow Service is started'),
  error => logger('ERROR', 'Workflow Service initialization error: ' + error)
);

// Enable Cross Origin Resource Sharing 
app.use('*', cors());

// Authorization: a user authentication is required
app.use((req, res, next) => {
  if (!getAuthUserName(req)) {
    logger('ERROR', 'Unauthorized: authenticated username is missing');
    res.status(401);
    res.send('Unauthorized: Access is denied due to missing credentials');
  }else {
    next();
  }
});

// Print Schema endpoint
app.use(rootUrl + '/schema', (req, res) => {
  res.set('Content-Type', 'text/plain');
  res.send(schemaPrinter.printSchema(schema));
});

// Print Introspection Schema endpoint
app.use(rootUrl + '/ischema', (req, res) => {
  res.set('Content-Type', 'text/plain');
  res.send(schemaPrinter.printIntrospectionSchema(schema));
});

// Stop node app
if (config.graphql.debugEnabled) {
  app.use(rootUrl + '/stop', (req, res) => {
    logger('INFO', 'Stop request');
    res.send('Stop request initiated');
    process.exit();
  });
}

// GraphQL endpoint
app.use(rootUrl, graphqlExpress(request => {
  const startTime = Date.now();

  request.uuid = uuidV1();
  request.workflow = {
    service: workflowService,
    context: getWorkflowContext(request)
  };
  request.loaders = createLoaders(config.loaders, request);
  request.resolverCount = 0;
  request.logTimeoutError = true;
  
  logger('INFO', 'new request ' + request.uuid + ' by ' + request.workflow.context.authUserName);

  request.incrementResolverCount =  function () {
    var runTime = Date.now() - startTime;
    if (runTime > config.graphql.queryTimeout) {
      if (request.logTimeoutError) {
        logger('ERROR', 'Request ' + request.uuid + ' query execution timeout');
      }
      request.logTimeoutError = false;
      throw('Query execution has timeout. Field resolution aborted');
    }
    this.resolverCount++;
  };

  return !config.graphql.debugEnabled ?
    {
      schema: schema,
      context: request,
      graphiql: config.graphql.graphiqlEnabled
    } :
    {
      schema: schema,
      context: request,
      graphiql: config.graphql.graphiqlEnabled,
      formatError: error => ({
        message: error.message,
        locations: error.locations,
        stack: error.stack
      }),
      extensions({ document, variables, operationName, result }) {
        return {
          requestId: request.uuid,
          runTime: Date.now() - startTime,
          resolverCount: request.resolverCount,
          operationCount: request.workflow.context.operationCount,
          operationErrorCount: request.workflow.context.operationErrorCount
        };
      }
    };
}));

// Start Express Server
app.listen(config.app.port, function () {
  logger('INFO', 'GraphQL Server running on port ' + config.app.port);
});
