/**
 * server.js
 * Server initialization
 */
'use strict';
const express = require('express');
const bodyParser = require('body-parser');
const cors = require('cors');
const uuidV1 = require('uuid/v1');
const WorkflowService = require('./services/workflow/workflowservice');
const { logger } = require('./utils/logger.js');
const config = require('./config');

const rootUrl = '/*/hooks';
const app = express();

logger('INFO', 'Application NODE_ENV: ' + process.env.NODE_ENV);
logger('INFO', 'Hooks debug enabled: ' + config.hooks.debugEnabled);
logger('INFO', 'Workflow service debug enabled: ' + config.workflowService.debugEnabled);

// Workflow service creation
const workflowService = new WorkflowService(config.workflowService);
workflowService.start().then(
  result => logger('INFO', 'Workflow Service is started'),
  error => logger('ERROR', 'Workflow Service initialization error: ' + error)
);

// Enable Cross Origin Resource Sharing 
app.use('*', cors());

// Stop node app
if (config.hooks.debugEnabled) {
  app.use(rootUrl + '/stop', (req, res) => {
    logger('INFO', 'Stop request');
    res.send('Stop request initiated');
    process.exit();
  });
}

// Parse application/x-www-form-urlencoded
app.use(bodyParser.urlencoded({ extended: false }));

// Parse application/json
app.use(bodyParser.json());

// Catch json syntax error
app.use(function (error, req, res, next) {
  if (error instanceof SyntaxError) {
    res.status(400);
    logger('ERROR', 'Bad request: invalid JSON');
    res.send({ error: 'Bad request: invalid JSON' });
  } else {
    next();
  }
});

// Hooks POST endpoint
app.post(rootUrl + '/:token' , (req, res) => {
  res.set('Content-Type', 'application/json');

  req.uuid = uuidV1();
  logger('INFO', 'New request ' + req.uuid );
  if (!req.body) {
    logger('ERROR', 'Request ' + req.uuid + ' missing JSON body');
    res.status(400);
    res.send({ error: 'A JSON body is required'});
    return;
  }
  const token = req.params.token;
  if (!token) {
    logger('ERROR', 'Request ' + req.uuid + ' missing token');
    res.status(400);
    res.send({ error: 'A token is required'});
    return;
  }
  var context = { 
    hookToken: token,
    operationCount: 0,
    operationErrorCount: 0
  };
  var body;

  // application/x-www-form-urlencoded
  if (req.body.payload) {
    try {
      body = JSON.parse(req.body.payload);
    } catch (e) {
      logger('ERROR', 'Bad request: invalid JSON payload');
      res.send({ error: 'Bad request: invalid JSON payload' });
      return;
    }
  }
  // application/json 
  else {
    body = req.body;
  }
  logger('INFO', 'Request ' + req.uuid + ' token:' + token + ' operation:' + body.operation);
  if (config.hooks.operations.indexOf(body.operation) < 0) {
    logger('ERROR', 'Request ' + req.uuid + ' operation is not allowed');
    res.status(403);
    res.send({ error: 'The requested operation is not allowed'});
    return;
  }

  workflowService.invoke(body.operation, context, body.args).then(
    result => res.send(result),
    error => {
      logger('ERROR', 'Request ' + req.uuid + ' invoke operation error');
      res.send({error: error });
    }
  );
});

// Start Express Server
app.listen(config.app.port, function () {
  logger('INFO', 'Hooks Server running on port ' + config.app.port);
});
