/**
 * workflowservice.js
 * Workflow Service
 */
'use strict';
const edge = require('edge');

// Use bluebird Promise to fix:
// https://github.com/tjanczuk/edge/issues/485
// https://github.com/Microsoft/vscode/issues/1746
const Promise = require('bluebird'); 

/**
 *  Represents a Workflow Service
 *  @class WorkflowService
 */
class WorkflowService {

  /**
   * Creates a new instance of WorkflowService.
   * @constructor
   * @param {Object} config A service configuration.
   */
  constructor (config) {
    this._config = config;

    // Load the workflow service assembly 
    this._createWorkflowServiceFacade = edge.func({
      assemblyFile: this._config.serviceAssemblyFilePath,
      typeName: 'Advantys.Workflow.ServiceFacade',
      methodName: 'CreateProxyAsync'
    });
  }

  /**
   * Starts the workflow service.
   * @return {Promise<Object>} A promise.
   */
  start () {
    const self = this;
    return new Promise(function (resolve, reject) {
      self._createWorkflowServiceFacade(self._config, function (error, result) {

        if (error) {
          if (self._config.debugEnabled) {
            self._config.logHandler('Workflow Service: start: error: ' + error);
          }
          reject(error);
        } else {
          self._workflowFacade = result;
          resolve();
        }
      });
    });
  }

  /**
   * Invokes a workfow operation asynchronously.
   * @param {String} operation An operation to execute.
   * @param {Object} context A workflow context.
   * @param {Object} args Arguments.
   * @return {Promise<Object>} A promise.
   */
  invoke (operation, context, args) {
    const self = this;

    return new Promise(function (resolve, reject) {
      if (!self._workflowFacade) {
        reject('Workflow Service is not started');
        return;
      }
      if (context.operationCount == self._config.maxOperationCount) {
        reject('The maximum number of operation per request has been reached');
        return;
      }
      context.operationCount++;
      var operationContext = {};
      if (context.authUserName) {
        operationContext['authUserName'] = context.authUserName;
      }
      if (context.impersonateUserName) {
        operationContext['impersonateUserName'] = context.impersonateUserName;
      }
      if (context.hookToken) {
        operationContext['hookToken'] = context.hookToken;
      }
      self._workflowFacade.invoke(
        {
          operation: operation,
          context: operationContext,
          args: args
        },
        function (error, result) {
          if (error || result.error) {
            if (self._config.debugEnabled) {
              self._config.logHandler('Workflow Service: invoke: error: ' + (error || result.error));
            }
            context.operationErrorCount++;
            reject(error || result.error);
          } else {
            resolve(result.data);
          }
        });
    });
  }

}

module.exports = WorkflowService;
